@@ -25,6 +25,8 @@
 #include <math.h>
 #include <stdbool.h>
 
+#include "linops/linop.h"
+#include "linops/someops.h"
 #include "num/multind.h"
 #include "num/fft.h"
 #include "num/flpmath.h"
@@ -62,7 +64,7 @@
 #endif
 
 
-static void eigen_herm3(int M, int N, float val[M], complex float matrix[N][N]) // ordering might be different to herm2
+static void eigen_herm3(int M, int N, float val[M], complex float matrix[N][N], int num_orthiter) // ordering might be different to herm2
 {
 	complex float mout[M][N];
 
@@ -71,67 +73,89 @@ static void eigen_herm3(int M, int N, float val[M], complex float matrix[N][N])
 			matrix[lj][li] = conj(matrix[li][lj]);
 
 	//mat_identity(M, N, mout);
-	orthiter(M, N, 30, val, mout, matrix);
+	orthiter(M, N, num_orthiter, val, mout, matrix);
 
 	for (int i = 0; i < M; i++)
 		for (int j = 0; j < N; j++)
 			matrix[i][j] = mout[i][j];
 }
 
-static float scurve(float x)
+
+
+static void md_scurve(int N, const long dims[N], float* dst, const float* src)
 {
-	if (x <= -1.)
-		return 0.;
+	float* tmp1 = md_alloc_sameplace(N, dims, FL_SIZE, src);
+	float* tmp2 = md_alloc_sameplace(N, dims, FL_SIZE, src);
 
-	if (x >= 1.)
-		return 1.;
+	md_sgreatequal(N, dims, tmp1, src, -1.);
+	md_slessequal(N, dims, tmp2, src, 1.);
 
-	return 0.5 * (1. + 2. * x / (1. + powf(x, 2.)));
+	md_mul(N, dims, tmp2, src, src);
+	md_sadd(N, dims, tmp2, tmp2, 1.);
+	md_div(N, dims, tmp2, src, tmp2);
+	md_sadd(N, dims, tmp2, tmp2, 0.5);
+
+	md_sgreatequal(N, dims, dst, src, 1.);
+	md_fmac(N, dims, dst, tmp1, tmp2);
+
+	md_free(tmp1);
+	md_free(tmp2);
 }
 
-static float crop_weight_function(float crth, float val)
+static void md_crop_weight_fun(int N, const long dims[N], float crth, complex float* dst, const complex float* src)
 {
-	return scurve((sqrtf(val) - crth) / (1. - crth));
+	md_zabs(N, dims, dst, src);
+
+	float* tmp = md_alloc_sameplace(N, dims, FL_SIZE, src);
+
+	md_real(N, dims, tmp, dst);
+
+	md_sqrt(N, dims, tmp, tmp);
+	md_sadd(N, dims, tmp, tmp, - crth);
+	md_smul(N, dims, tmp, tmp, 1. / (1. - crth));
+	md_scurve(N, dims, tmp, tmp);
+
+	md_zcmpl_real(N, dims, dst, tmp);
+
+	md_free(tmp);
 }
 
-static float crop_thresh_function(float crth, float val)
+static void md_crop_thresh_fun(int N, const long dims[N], float crth, complex float* dst, const complex float* src)
 {
-	return (val <= crth) ? 0. : 1.;
+	md_zabs(N, dims, dst, src);
+	md_zsgreatequal(N, dims, dst, dst, crth);
 }
 
-typedef float (*weight_function)(float crth, float val);
 
-static void crop_weight(const long dims[DIMS], complex float* ptr, weight_function fun, float crth, const complex float* map)
+typedef void (*md_weight_function)(int N, const long dims[N], float crth, complex float* dst, const complex float* src);
+
+static void md_crop_weight(int N, const long dims[N], complex float* ptr, md_weight_function fun, float crth, const complex float* map)
 {
-	long xx = dims[0];
-	long yy = dims[1];
-	long zz = dims[2];
-	long cc = dims[3];
-	long mm = dims[4];
+	assert(4 < N);
 
-	assert(DIMS > 5);
-	assert(1 == md_calc_size(DIMS - 5, dims + 5));
+	long wgh_dims[N];
+	md_select_dims(N, FFT_FLAGS | MAPS_FLAG, wgh_dims, dims);
 
-	for (long m = 0; m < mm; m++) {
-#pragma omp parallel for
-		for (long k = 0; k < zz; k++) {
-			for (long i = 0; i < yy; i++) {
-				for (long j = 0; j < xx; j++) {
+	complex float* tmp = md_alloc_sameplace(N, wgh_dims, CFL_SIZE, map);
 
-					float val = cabsf(map[((m * zz + k) * yy + i) * xx + j]);
+	fun(N, wgh_dims, crth, tmp, map);
 
-					for (long c = 0; c < cc; c++)
-						ptr[(((m * cc + c) * zz + k) * yy + i) * xx + j] *= fun(crth, val);
-				}
-			}
-		}
-	}
+	long strs[N];
+	long wgh_strs[N];
+
+	md_calc_strides(N, strs, dims, CFL_SIZE);
+	md_calc_strides(N, wgh_strs, wgh_dims, CFL_SIZE);
+
+	md_zmul2(N, dims, strs, ptr, strs, ptr, wgh_strs, tmp);
+
+	md_free(tmp);
 }
 
 
+
 void crop_sens(const long dims[DIMS], complex float* ptr, bool soft, float crth, const complex float* map)
 {
-	crop_weight(dims, ptr, soft ? crop_weight_function : crop_thresh_function, crth, map);
+	md_crop_weight(DIMS, dims, ptr, soft ? md_crop_weight_fun : md_crop_thresh_fun, crth, map);
 }
 
 
@@ -148,8 +172,11 @@ void crop_sens(const long dims[DIMS], complex float* ptr, bool soft, float crth,
  *	calreg_dims     - Dimension of the calibration region.
  *	calreg	        - Calibration data.
  */
-static float sure_crop(float var, const long evec_dims[5], complex float* evec_data, complex float* eptr, const long calreg_dims[5], const complex float* calreg)
+static float sure_crop(float var, const long evec_dims[DIMS], complex float* evec_data, complex float* eptr, const long calreg_dims[DIMS], const complex float* calreg)
 {
+	assert(1 == md_calc_size(DIMS - 5, evec_dims + 5));
+	assert(1 == md_calc_size(DIMS - 5, calreg_dims + 5));
+
 	long num_maps = evec_dims[4];
 
 	// Construct low-resolution image
@@ -161,7 +188,10 @@ static float sure_crop(float var, const long evec_dims[5], complex float* evec_d
 	md_clear(5, im_dims, im, CFL_SIZE);
 	md_resize_center(5, im_dims, im, calreg_dims, calreg, CFL_SIZE);
 
-	ifftuc(5, im_dims, FFT_FLAGS, im, im);
+	auto lop_fft_im = linop_fftc_create(5, im_dims, FFT_FLAGS);
+	auto lop_fft_evec = linop_fftc_create(5, evec_dims, FFT_FLAGS);
+
+	linop_adjoint(lop_fft_im, 5, im_dims, im, 5, im_dims, im);
 
 	// Temporary vector for crop dimensions
 	long cropdims[5];
@@ -227,7 +257,7 @@ static float sure_crop(float var, const long evec_dims[5], complex float* evec_d
 	long tdims_ip[5];
 	long tdims_proj[5];
 
-	for (unsigned int i = 0; i < 5; i++) {
+	for (int i = 0; i < 5; i++) {
 
 		assert((im_dims[i] == evec_dims[i]) || (1 == im_dims[i]) || (1 == evec_dims[i]));
 		assert((W_dims[i] == evec_dims[i]) || (1 == W_dims[i]) || (1 == evec_dims[i]));
@@ -273,31 +303,40 @@ static float sure_crop(float var, const long evec_dims[5], complex float* evec_d
 			old_mse = mse;
 			mse = 0.;
 
-			crop_weight(evec_dims, M, crop_thresh_function, c, W);
+			md_crop_weight(5, evec_dims, M, md_crop_thresh_fun, c, W);
 
 			md_zfmacc2(5, tdims_ip, stro_ip, ip, str1_ip, im, str2_ip, M); // Projection.
 			md_zfmac2(5, tdims_proj, stro_proj, proj, str1_proj, ip, str2_proj, M);
 
-			fftuc(5, im_dims, FFT_FLAGS, proj, proj);                                // Low res proj img.
+			linop_forward(lop_fft_im, 5, im_dims, proj, 5, im_dims, proj);		// Low res proj img.                             
 
 			md_resize_center(5, calreg_dims, TC, im_dims, proj, CFL_SIZE);
 			md_resize_center(5, im_dims, proj, calreg_dims, TC, CFL_SIZE);
 
-			ifftuc(5, im_dims, FFT_FLAGS, proj, proj);
+			linop_adjoint(lop_fft_im, 5, im_dims, proj, 5, im_dims, proj);
 
+#if 1
+			complex float* diff = md_alloc_sameplace(5, im_dims, CFL_SIZE, im);
+			md_zsub(5, im_dims, diff, im, proj);
+			mse += powf(md_znorm(5, im_dims, diff), 2);
+			md_free(diff);
+#else
 			for (long jdx = 0; jdx < md_calc_size(5, im_dims); jdx++)
 				mse += powf(cabsf(im[jdx] - proj[jdx]), 2.);
+#endif
 
-			fftuc(5, evec_dims, FFT_FLAGS, LM, M);                                   // low-res maps .
+			linop_forward(lop_fft_evec, 5, evec_dims, LM, 5, evec_dims, M);		// low-res maps .                       
 
 			md_resize_center(5, cropdims, CM, evec_dims, LM, CFL_SIZE);
 			md_resize_center(5, evec_dims, LM, cropdims, CM, CFL_SIZE);
 
-			ifftuc(5, evec_dims, FFT_FLAGS, LM, LM);
+			linop_adjoint(lop_fft_evec, 5, evec_dims, LM, 5, evec_dims, LM);
 
 			md_zfmacc2(5, evec_dims, stro_div, div, str1_div, LM, str2_div, LM);     // Calc SURE div using low res maps.
 
-			mse += 2. * var * crealf(*div);
+			complex float div_cpu;
+			md_copy(1, MD_DIMS(1), &div_cpu, div, CFL_SIZE);
+			mse += 2. * var * crealf(div_cpu);
 
 			if (ctr2 == 1)
 				debug_printf(DP_INFO, "| %4ld | %4ld | %0.4f | %0.12e |\n", ctr1, ctr2, c, mse);
@@ -313,6 +352,9 @@ static float sure_crop(float var, const long evec_dims[5], complex float* evec_d
 		c += s;
 	}
 
+	linop_free(lop_fft_im);
+	linop_free(lop_fft_evec);
+
 	c = c + s;
 
 	debug_printf(DP_INFO, "---------------------------------------------\n");
@@ -335,7 +377,7 @@ static float sure_crop(float var, const long evec_dims[5], complex float* evec_d
 
 
 
-void calone(const struct ecalib_conf* conf, const long cov_dims[4], complex float* imgcov, unsigned int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data)
+void calone(const struct ecalib_conf* conf, const long cov_dims[4], complex float* imgcov, int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data)
 {
 	assert(1 == md_calc_size(DIMS - 5, calreg_dims + 5));
 
@@ -378,14 +420,14 @@ void calone(const struct ecalib_conf* conf, const long cov_dims[4], complex floa
 /* calculate point-wise maps 
  *
  */
-void eigenmaps(const long out_dims[DIMS], complex float* optr, complex float* eptr, const complex float* imgcov2, const long msk_dims[3], const bool* msk, bool orthiter, bool ecal_usegpu)
+void eigenmaps(const long out_dims[DIMS], complex float* optr, complex float* eptr, const complex float* imgcov2, const long msk_dims[3], const bool* msk, bool orthiter, int num_orthiter, bool ecal_usegpu)
 {
 #ifdef USE_CUDA
 	if (ecal_usegpu) {
 
 		//FIXME cuda version should be able to return sensitivities for a subset of image-space points
 		assert(!msk);
-		eigenmapscu(out_dims, optr, eptr, imgcov2);
+		eigenmapscu(out_dims, optr, eptr, imgcov2, num_orthiter);
 		return;
 	}
 #else
@@ -432,7 +474,7 @@ void eigenmaps(const long out_dims[DIMS], complex float* optr, complex float* ep
 					unpack_tri_matrix(channels, cov, tmp);
 
 					if (orthiter) 
-						eigen_herm3(maps, channels, val, cov);
+						eigen_herm3(maps, channels, val, cov, num_orthiter);
 					else 
 						lapack_eig(channels, val, cov);
 
@@ -462,7 +504,6 @@ void eigenmaps(const long out_dims[DIMS], complex float* optr, complex float* ep
 
 void caltwo(const struct ecalib_conf* conf, const long out_dims[DIMS], complex float* out_data, complex float* emaps, const long in_dims[4], complex float* in_data, const long msk_dims[3], const bool* msk)
 {
-
 	long xx = out_dims[0];
 	long yy = out_dims[1];
 	long zz = out_dims[2];
@@ -497,7 +538,7 @@ void caltwo(const struct ecalib_conf* conf, const long out_dims[DIMS], complex f
 
 	debug_printf(DP_DEBUG1, "Point-wise eigen-decomposition...\n");
 
-	eigenmaps(out_dims, out_data, emaps, imgcov2, msk_dims, msk, conf->orthiter, conf->usegpu);
+	eigenmaps(out_dims, out_data, emaps, imgcov2, msk_dims, msk, conf->orthiter, conf->num_orthiter, conf->usegpu);
 
 	md_free(imgcov2);
 }
@@ -519,14 +560,31 @@ void calone_dims(const struct ecalib_conf* conf, long cov_dims[4], long channels
 
 
 
-const struct ecalib_conf ecalib_defaults = { { 6, 6, 6 }, 0.001, -1, -1., false, false, 0.8, true, false, -1., false, true, -1., false};
+const struct ecalib_conf ecalib_defaults = {
+
+	.kdims = {6, 6, 6},
+	.threshold = 0.001,
+	.numsv = -1,
+	.percentsv = -1.,
+	.weighting = false,
+	.softcrop = false,
+	.crop = 0.8,
+	.orthiter = true,
+	.num_orthiter = 30,
+	.usegpu = false,
+	.perturb = -1.,
+	.intensity = false,
+	.rotphase = true,
+	.var = -1.,
+	.automate = false,
+};
 
 
 
 
 
 
-void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex float* out_data, complex float* eptr, unsigned int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data, const long msk_dims[3], const bool* msk)
+void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex float* out_data, complex float* eptr, int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data, const long msk_dims[3], const bool* msk)
 {
 	long channels = calreg_dims[3];
 	long maps = out_dims[4];
@@ -549,8 +607,8 @@ void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex f
 
 	} else {
 
-		for (unsigned int i = 0; i < channels; i++)
-			for (unsigned int j = 0; j < channels; j++)
+		for (int i = 0; i < channels; i++)
+			for (int j = 0; j < channels; j++)
 				rot[i][j] = (i == j) ? 1. : 0.;
 	}
 
@@ -583,7 +641,17 @@ void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex f
 		md_zsmul(DIMS, out_dims, out_data, out_data, sqrtf((float)channels));
 	}
 
-	float c = (conf->crop >= 0.) ? conf->crop : sure_crop(conf->var, out_dims, out_data, eptr, calreg_dims, data);
+
+	const complex float* data_tmp = data;
+#ifdef USE_CUDA
+	if (conf->usegpu)
+		data_tmp = md_gpu_move(DIMS, calreg_dims, data, CFL_SIZE);
+#endif
+
+	float c = (conf->crop >= 0.) ? conf->crop : sure_crop(conf->var, out_dims, out_data, eptr, calreg_dims, data_tmp);
+
+	if (data_tmp != data)
+		md_free(data_tmp);
 
 	debug_printf(DP_DEBUG1, "Crop maps... (c = %.2f)\n", c);
 
@@ -599,7 +667,7 @@ void calib2(const struct ecalib_conf* conf, const long out_dims[DIMS], complex f
 
 
 
-void calib(const struct ecalib_conf* conf, const long out_dims[DIMS], complex float* out_data, complex float* eptr, unsigned int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data)
+void calib(const struct ecalib_conf* conf, const long out_dims[DIMS], complex float* out_data, complex float* eptr, int SN, float svals[SN], const long calreg_dims[DIMS], const complex float* data)
 {
 	calib2(conf, out_dims, out_data, eptr, SN, svals, calreg_dims, data, NULL, NULL);
 }
@@ -633,9 +701,10 @@ static void perturb(const long dims[2], complex float* vecs, float amt)
 }
 
 
-static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, const float val[N])
+static int number_of_kernels(const struct ecalib_conf* conf, int N, const float val[N])
 {
-	unsigned int n = 0;
+	int n = 0;
+
 	if (-1 != conf->numsv) {
 
 		n = conf->numsv;
@@ -644,7 +713,7 @@ static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, con
 
 	} else if (conf->percentsv != -1.) {
 
-		n = (unsigned int)(N * conf->percentsv / 100.);
+		n = N * conf->percentsv / 100.;
 		assert(-1 == conf->numsv);
 		assert(-1. == conf->threshold);
 
@@ -653,11 +722,9 @@ static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, con
 		assert(-1 == conf->numsv);
 		assert(-1. == conf->percentsv);
 
-		for (unsigned int i = 0; i < N; i++) {
-
+		for (int i = 0; i < N; i++)
 			if (val[i] / val[0] > sqrtf(conf->threshold))
 				n++;
-		}
 	}
 
 	if (val[0] <= 0.)
@@ -666,9 +733,11 @@ static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, con
 	debug_printf(DP_DEBUG1, "Using %d/%ld kernels (%.2f%%, last SV: %f%s).\n", n, N, (float)n / (float)N * 100., (n > 0) ? (val[n - 1] / val[0]) : 1., conf->weighting ? ", weighted" : "");
 
 	float tr = 0.;
-	for (unsigned int i = 0; i < N; i++) {
+
+	for (int i = 0; i < N; i++) {
 
 		tr += powf(val[i], 2.);
+
 		debug_printf(DP_DEBUG3, "SVALS %f (%f)\n", val[i], val[i] / val[0]);
 	}
 
@@ -679,7 +748,7 @@ static int number_of_kernels(const struct ecalib_conf* conf, unsigned int N, con
 }
 
 
-void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], complex float** nskerns_ptr, unsigned int SN, float val[SN], const long caldims[DIMS], const complex float* caldata)
+void compute_kernels(const struct ecalib_conf* conf, long nskerns_dims[5], complex float** nskerns_ptr, int SN, float val[SN], const long caldims[DIMS], const complex float* caldata)
 {
 	assert(1 == md_calc_size(DIMS - 5, caldims + 5));
 
