@@ -1,5 +1,5 @@
 /* Copyright 2014-2017. The Regents of the University of California.
- * Copyright 2018. Martin Uecker.
+ * Copyright 2018-2022. Martin Uecker.
  * All rights reserved. Use of this source code is governed by 
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -18,13 +18,14 @@ struct linop_s;
 
 struct reg_s {
 
-	enum { L1WAV, NIHTWAV, NIHTIM, TV, LLR, MLR, IMAGL1, IMAGL2, L1IMG, L2IMG, FTL1, LAPLACE, POS } xform;
+	enum { L1WAV, NIHTWAV, NIHTIM, TV, LLR, MLR, IMAGL1, IMAGL2, L1IMG, L2IMG, FTL1, LAPLACE, POS, TENFL, TGV, ICTV } xform;
 
 	unsigned int xflags;
 	unsigned int jflags;
 
 	float lambda;
 	unsigned int k;
+	const char* graph_file;
 };
 
 
@@ -32,8 +33,9 @@ struct opt_reg_s {
 
 	float lambda;
 	struct reg_s regs[NUM_REGS];
-	unsigned int r;
-	unsigned int k;
+	int r;
+	int svars;
+	int sr;
 };
 
 
@@ -42,7 +44,7 @@ extern _Bool opt_reg_init(struct opt_reg_s* ropts);
 
 extern void opt_bpursuit_configure(struct opt_reg_s* ropts, const struct operator_p_s* prox_ops[NUM_REGS], const struct linop_s* trafos[NUM_REGS], const struct linop_s* model_op, const _Complex float* data, const float eps);
 
-extern void opt_reg_configure(unsigned int N, const long img_dims[__VLA(N)], struct opt_reg_s* ropts, const struct operator_p_s* prox_ops[NUM_REGS], const struct linop_s* trafos[NUM_REGS], unsigned int llr_blk, unsigned int shift_mode, _Bool use_gpu);
+extern void opt_reg_configure(int N, const long img_dims[__VLA(N)], struct opt_reg_s* ropts, const struct operator_p_s* prox_ops[NUM_REGS], const struct linop_s* trafos[NUM_REGS], unsigned int llr_blk, unsigned int shift_mode, const char* wtype_str, _Bool use_gpu);
 
 extern void opt_reg_free(struct opt_reg_s* ropts, const struct operator_p_s* prox_ops[NUM_REGS], const struct linop_s* trafos[NUM_REGS]);
 
