@@ -16,25 +16,36 @@
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
+#include "misc/opts.h"
 
 #ifndef DIMS
 #define DIMS 16
 #endif
 
-static const char usage_str[] = "<input> <output>";
-static const char help_str[] = "Invert array (1 / <input>). The output is set to zero in case of divide by zero.\n";
+static const char help_str[] = "Invert array (1 / <input>). The output is set to zero in case of divide by zero.";
 
 
 int main_invert(int argc, char* argv[argc])
 {
-	mini_cmdline(&argc, argv, 2, usage_str, help_str);
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
+	const struct opt_s opts[] = { };
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	long dims[DIMS];
 
-	complex float* idata = load_cfl(argv[1], DIMS, dims);
-	complex float* odata = create_cfl(argv[2], DIMS, dims);
+	complex float* idata = load_cfl(in_file, DIMS, dims);
+	complex float* odata = create_cfl(out_file, DIMS, dims);
 		
 	#pragma omp parallel for
 	for (long i = 0; i < md_calc_size(DIMS, dims); i++)
