@@ -1,7 +1,3 @@
-/* Copyright 2014. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by
- * a BSD-style license which can be found in the LICENSE file.
- */
 
 #ifndef __SOMEOPS_H
 #define __SOMEOPS_H
@@ -12,6 +8,10 @@
 
 extern struct linop_s* linop_cdiag_create(unsigned int N, const long dims[__VLA(N)], unsigned int flags, const _Complex float* diag);
 extern struct linop_s* linop_rdiag_create(unsigned int N, const long dims[__VLA(N)], unsigned int flags, const _Complex float* diag);
+extern void linop_gdiag_set_diag(const struct linop_s* lop, int N, const long ddims[__VLA(N)], const _Complex float* diag);
+extern struct linop_s* linop_scale_create(unsigned int N, const long dims[N], const _Complex float scale);
+extern struct linop_s* linop_zconj_create(unsigned int N, const long dims[N]);
+extern struct linop_s* linop_zreal_create(unsigned int N, const long dims[N]);
 
 extern struct linop_s* linop_identity_create(unsigned int N, const long dims[__VLA(N)]);
 
@@ -20,6 +20,7 @@ extern struct linop_s* linop_resize_center_create(unsigned int N, const long out
 extern struct linop_s* linop_expand_create(unsigned int N, const long out_dims[__VLA(N)], const long in_dims[__VLA(N)]);
 extern struct linop_s* linop_reshape_create(unsigned int A, const long out_dims[__VLA(A)], int B, const long in_dims[__VLA(B)]);
 extern struct linop_s* linop_extract_create(unsigned int N, const long pos[N], const long out_dims[N], const long in_dims[N]);
+extern struct linop_s* linop_permute_create(unsigned int N, const unsigned int order[__VLA(N)], const long idims[N]);
 extern struct linop_s* linop_transpose_create(int N, int a, int b, const long dims[N]);
 
 
@@ -31,6 +32,13 @@ extern struct linop_s* linop_fft_create_measure(int N, const long dims[__VLA(N)]
 
 extern struct linop_s* linop_cdf97_create(int N, const long dims[__VLA(N)], unsigned int flag);
 
+#ifndef __PADD_ENUMS
+#define __PADD_ENUMS
+enum PADDING { PAD_VALID, PAD_SAME, PAD_CYCLIC, PAD_SYMMETRIC, PAD_REFLECT, PAD_CAUSAL };
+#endif
+extern struct linop_s* linop_padding_create_onedim(int N, const long dims[N], enum PADDING pad_type, int pad_dim, long pad_for, long pad_after);
+extern struct linop_s* linop_padding_create(int N, const long dims[N], enum PADDING pad_type, long pad_for[N], long pad_after[N]);
+
 #ifndef __CONV_ENUMS
 #define __CONV_ENUMS
 enum conv_mode { CONV_SYMMETRIC, CONV_CAUSAL, CONV_ANTICAUSAL };
@@ -48,4 +56,3 @@ extern struct linop_s* linop_matrix_chain(const struct linop_s* a, const struct
 
 #include "misc/cppwrap.h"
 #endif // __SOMEOPS_H
-
