@@ -17,9 +17,10 @@ typedef void (*operator_fun_t)(const operator_data_t* _data, unsigned int N, voi
 typedef void (*operator_del_t)(const operator_data_t* _data);
 
 
-
+struct graph_s;
 struct operator_s;
 
+typedef const struct graph_s* (*operator_get_graph_t)(const struct operator_s*);
 
 // create functions
 
@@ -34,12 +35,11 @@ extern const struct operator_s* operator_create2(unsigned int ON, const long out
 
 extern const struct operator_s* operator_generic_create(unsigned int N, const _Bool io_flags[N],
 		const unsigned int D[__VLA(N)], const long* out_dims[__VLA(N)],
-		operator_data_t* data, operator_fun_t apply, operator_del_t del);
+		operator_data_t* data, operator_fun_t apply, operator_del_t del, operator_get_graph_t get_graph);
 
 extern const struct operator_s* operator_generic_create2(unsigned int N, const _Bool io_flags[N],
 			const unsigned int D[__VLA(N)], const long* out_dims[__VLA(N)], const long* out_strs[__VLA(N)],
-			operator_data_t* data, operator_fun_t apply, operator_del_t del);
-
+			operator_data_t* data, operator_fun_t apply, operator_del_t del, operator_get_graph_t get_graph);
 
 
 extern const struct operator_s* operator_identity_create(unsigned int N, const long dims[__VLA(N)]);
@@ -81,11 +81,13 @@ extern const struct operator_s* operator_unref(const struct operator_s* x);
 
 // apply functions
 extern void operator_generic_apply_unchecked(const struct operator_s* op, unsigned int N, void* args[__VLA(N)]);
+extern void operator_generic_apply_parallel_unchecked(int D, const struct operator_s* op[__VLA(D)], unsigned int N, void* args[__VLA(D)][N], int num_threads);
 extern void operator_apply(const struct operator_s* op, unsigned int ON, const long odims[__VLA(ON)], _Complex float* dst, const long IN, const long idims[__VLA(IN)], const _Complex float* src);
 extern void operator_apply2(const struct operator_s* op, unsigned int ON, const long odims[__VLA(ON)], const long ostrs[__VLA(ON)], _Complex float* dst, const long IN, const long idims[__VLA(IN)], const long istrs[__VLA(IN)], const _Complex float* src);
 
 extern void operator_apply_unchecked(const struct operator_s* op, _Complex float* dst, const _Complex float* src);
-extern void operator_apply_parallel_unchecked(unsigned int N, const struct operator_s* op[__VLA(N)], _Complex float* dst[__VLA(N)], const _Complex float* src);
+extern void operator_apply_parallel_unchecked(int D, const struct operator_s* op[__VLA(D)], _Complex float* dst[__VLA(D)], const _Complex float* src[__VLA(D)], int num_threads);
+extern void operator_apply_joined_unchecked(unsigned int N, const struct operator_s* op[__VLA(N)], _Complex float* dst[__VLA(N)], const _Complex float* src);
 
 
 // get functions
@@ -100,12 +102,16 @@ extern const struct iovec_s* operator_arg_out_codomain(const struct operator_s*
 extern const struct iovec_s* operator_domain(const struct operator_s* op);
 extern const struct iovec_s* operator_codomain(const struct operator_s* op);
 
+enum debug_levels;
+void operator_debug(enum debug_levels dl, const struct operator_s* x);
+
 extern operator_data_t* operator_get_data(const struct operator_s* op);
 extern const _Bool* operator_get_io_flags(const struct operator_s* op);
 
+extern _Bool check_simple_copy(const struct operator_s* op);
 
 extern const struct operator_s* operator_copy_wrapper(unsigned int N, const long* strs[N], const struct operator_s* op);
-
+extern const struct operator_s* operator_copy_wrapper_sameplace(unsigned int N, const long* strs[N], const struct operator_s* op, const void* ref);
 
 extern const struct operator_s* operator_gpu_wrapper2(const struct operator_s* op, long move_flags);
 extern const struct operator_s* operator_gpu_wrapper(const struct operator_s* op);
@@ -131,7 +137,18 @@ extern const struct operator_s* operator_extract_create(const struct operator_s*
 extern const struct operator_s* operator_extract_create2(const struct operator_s* op, int a, int Da, const long dimsa[Da], const long strsa[Da], const long pos[Da]);
 extern const struct operator_s* operator_permute(const struct operator_s* op, int N, const int perm[N]);
 extern const struct operator_s* operator_reshape(const struct operator_s* op, unsigned int i, long N, const long dims[__VLA(N)]);
+extern const struct operator_s* get_in_reshape(const struct operator_s* op);
 
+extern const struct operator_s* operator_zadd_create(int II, int N, const long dims[__VLA(N)]);
+extern _Bool operator_is_zadd(const struct operator_s* op);
+
+
+extern _Bool operator_identify(const struct operator_s* a, const struct operator_s* b);
+
+extern struct list_s* operator_get_list(const struct operator_s* op);
+extern const struct graph_s* operator_get_graph(const struct operator_s* op);
+
+extern const struct operator_s* graph_optimize_operator_F(const struct operator_s* op);
 
 extern _Bool operator_zero_or_null_p(const struct operator_s* op);
 
