@@ -1,12 +1,12 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2015-2019. Martin Uecker.
+ * Copyright 2015-2022. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2019 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2014-2016 Frank Ong <frankong@berkeley.edu>
- * 2014-2018 Jon Tamir <jtamir@eecs.berkeley.edu>
+ * 2012-2022 Martin Uecker
+ * 2014-2016 Frank Ong
+ * 2014-2018 Jon Tamir
  */
 
 #include <assert.h>
@@ -28,6 +28,7 @@
 #include "linops/linop.h"
 #include "linops/fmac.h"
 #include "linops/sampling.h"
+#include "linops/someops.h"
 
 #include "noncart/nufft.h"
 
@@ -48,12 +49,14 @@
 #include "num/iovec.h"
 #include "num/ops.h"
 
-static const char usage_str[] = "<kspace> <sensitivities> <output>";
-static const char help_str[] = "Parallel-imaging compressed-sensing reconstruction.";
+static const char help_str[] = "Parallel-imaging compressed-sensing reconstruction.\n";
+                 
 
-
-
-static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS], const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf, const long wgs_dims[DIMS], const complex float* weights, const long basis_dims[DIMS], const complex float* basis, struct operator_s** precond_op)
+static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS],
+			const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf,
+			const long wgs_dims[DIMS], const complex float* weights,
+			const long basis_dims[DIMS], const complex float* basis,
+			const struct linop_s** fft_opp)
 {
 	long coilim_dims[DIMS];
 	long img_dims[DIMS];
@@ -63,24 +66,41 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 	long ksp_dims2[DIMS];
 	md_copy_dims(DIMS, ksp_dims2, ksp_dims);
 	ksp_dims2[COEFF_DIM] = max_dims[COEFF_DIM];
-	//ksp_dims2[TE_DIM] = 1;
 
 	debug_print_dims(DP_INFO, DIMS, ksp_dims2);
 	debug_print_dims(DP_INFO, DIMS, coilim_dims);
 
-	const struct linop_s* fft_op = nufft_create2(DIMS, ksp_dims2, coilim_dims, traj_dims, traj, wgs_dims, weights, basis_dims, basis, conf);
-	const struct linop_s* maps_op = maps2_create(coilim_dims, map_dims, img_dims, maps);
-	const struct linop_s* lop = linop_chain_FF(maps_op, fft_op);
+	const struct linop_s* nufft_op = nufft_create2(DIMS, ksp_dims2, coilim_dims,
+						traj_dims, traj,
+						(weights ? wgs_dims : NULL), weights,
+						(basis ? basis_dims : NULL), basis, conf);
 
-	//precond_op[0] = (struct operator_s*) nufft_precond_create( fft_op );
-	precond_op[0] = NULL;
+	const struct linop_s* maps_op = maps2_create(coilim_dims, map_dims, img_dims, maps);
+	const struct linop_s* lop = linop_chain(maps_op, nufft_op);
+
+	if (NULL != fft_opp)
+		*fft_opp = linop_clone(nufft_op);
+
+	linop_free(maps_op);
+	linop_free(nufft_op);
 
 	return lop;
 }
 
 
-int main_pics(int argc, char* argv[])
+int main_pics(int argc, char* argv[argc])
 {
+	const char* ksp_file = NULL;
+	const char* sens_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &ksp_file, "kspace"),
+		ARG_INFILE(true, &sens_file, "sensitivities"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	// Initialize default parameters
 
 	struct sense_conf conf = sense_defaults;
@@ -105,6 +125,8 @@ int main_pics(int argc, char* argv[])
 	float restrict_fov = -1.;
 	const char* pat_file = NULL;
 	const char* traj_file = NULL;
+	const char* psf_ifile = NULL;
+	const char* psf_ofile = NULL;
 	bool scale_im = false;
 	bool eigen = false;
 	float scaling = 0.;
@@ -113,6 +135,7 @@ int main_pics(int argc, char* argv[])
         bool sms = false;
 
 	unsigned int llr_blk = 8;
+	const char* wtype_str = "dau2";
 
 	const char* image_truth_file = NULL;
 	bool im_truth = false;
@@ -129,7 +152,7 @@ int main_pics(int argc, char* argv[])
 	bool hogwild = false;
 	bool fast = false;
 
-	unsigned int gpun = 0;
+	int gpun = -1;
 
 	struct opt_reg_s ropts;
 	opt_reg_init(&ropts);
@@ -138,18 +161,18 @@ int main_pics(int argc, char* argv[])
 
 	const struct opt_s opts[] = {
 
-		{ 'l', true, opt_reg, &ropts, "1/-l2\t\ttoggle l1-wavelet or l2 regularization." },
+		{ 'l', NULL, true, OPT_SPECIAL, opt_reg, &ropts, "\b1/-l2", "  toggle l1-wavelet or l2 regularization." },
 		OPT_FLOAT('r', &ropts.lambda, "lambda", "regularization parameter"),
-		{ 'R', true, opt_reg, &ropts, " <T>:A:B:C\tgeneralized regularization options (-Rh for help)" },
+		{ 'R', NULL, true, OPT_SPECIAL, opt_reg, &ropts, "<T>:A:B:C", "generalized regularization options (-Rh for help)" },
 		OPT_SET('c', &conf.rvc, "real-value constraint"),
 		OPT_FLOAT('s', &step, "step", "iteration stepsize"),
 		OPT_UINT('i', &maxiter, "iter", "max. number of iterations"),
-		OPT_STRING('t', &traj_file, "file", "k-space trajectory"),
+		OPT_INFILE('t', &traj_file, "file", "k-space trajectory"),
 		OPT_CLEAR('n', &randshift, "disable random wavelet cycle spinning"),
 		OPT_SET('N', &overlapping_blocks, "do fully overlapping LLR blocks"),
 		OPT_SET('g', &conf.gpu, "use GPU"),
-		OPT_UINT('G', &gpun, "gpun", "use GPU device gpun"),
-		OPT_STRING('p', &pat_file, "file", "pattern or weights"),
+		OPT_INT('G', &gpun, "gpun", "use GPU device gpun"),
+		OPT_INFILE('p', &pat_file, "file", "pattern or weights"),
 		OPT_SELECT('I', enum algo_t, &algo, ALGO_IST, "select IST"),
 		OPT_UINT('b', &llr_blk, "blk", "Lowrank block size"),
 		OPT_SET('e', &eigen, "Scale stepsize based on max. eigenvalue"),
@@ -157,8 +180,8 @@ int main_pics(int argc, char* argv[])
 		OPT_SET('D', &admm.dynamic_rho, "(ADMM dynamic step size)"),
 		OPT_SET('F', &fast, "(fast)"),
 		OPT_SET('J', &admm.relative_norm, "(ADMM residual balancing)"),
-		OPT_STRING('T', &image_truth_file, "file", "(truth file)"),
-		OPT_STRING('W', &image_start_file, "<img>", "Warm start with <img>"),
+		OPT_INFILE('T', &image_truth_file, "file", "(truth file)"),
+		OPT_INFILE('W', &image_start_file, "<img>", "Warm start with <img>"),
 		OPT_INT('d', &debug_level, "level", "Debug level"),
 		OPT_INT('O', &conf.rwiter, "rwiter", "(reweighting)"),
 		OPT_FLOAT('o', &conf.gamma, "gamma", "(reweighting)"),
@@ -167,18 +190,22 @@ int main_pics(int argc, char* argv[])
 		OPT_FLOAT('q', &conf.cclambda, "cclambda", "(cclambda)"),
 		OPT_FLOAT('f', &restrict_fov, "rfov", "restrict FOV"),
 		OPT_SELECT('m', enum algo_t, &algo, ALGO_ADMM, "select ADMM"),
-		OPT_FLOAT('w', &scaling, "val", "inverse scaling of the data"),
+		OPT_FLOAT('w', &scaling, "", "inverse scaling of the data"),
 		OPT_SET('S', &scale_im, "re-scale the image after reconstruction"),
 		OPT_UINT('L', &loop_flags, "flags", "batch-mode"),
 		OPT_SET('K', &nuconf.pcycle, "randshift for NUFFT"),
-		OPT_STRING('B', &basis_file, "file", "temporal (or other) basis"),
+		OPT_INFILE('B', &basis_file, "file", "temporal (or other) basis"),
 		OPT_FLOAT('P', &bpsense_eps, "eps", "Basis Pursuit formulation, || y- Ax ||_2 <= eps"),
 		OPT_SELECT('a', enum algo_t, &algo, ALGO_PRIDU, "select Primal Dual"),
 		OPT_SET('M', &sms, "Simultaneous Multi-Slice reconstruction"),
-		OPT_SET('U', &nuconf.lowmem, "Use low-mem mode of the nuFFT"),
+		OPTL_SET('U', "lowmem", &nuconf.lowmem, "Use low-mem mode of the nuFFT"),
+		OPTL_OUTFILE(0, "psf_export", &psf_ofile, "file", "Export PSF to file"),
+		OPTL_INFILE(0, "psf_import", &psf_ifile, "file", "Import PSF from file"),
+		OPTL_STRING(0, "wavelet", &wtype_str, "name", "wavelet type (haar,dau2,cdf44)"),
 	};
 
-	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	if (NULL != image_truth_file)
 		im_truth = true;
@@ -206,7 +233,7 @@ int main_pics(int argc, char* argv[])
 
 	// load kspace and maps and get dimensions
 
-	complex float* kspace = load_cfl(argv[1], DIMS, ksp_dims);
+	complex float* kspace = load_cfl(ksp_file, DIMS, ksp_dims);
 
         if (sms) {
 
@@ -218,7 +245,7 @@ int main_pics(int argc, char* argv[])
                 debug_printf(DP_INFO, "SMS reconstruction: MB = %ld\n", ksp_dims[SLICE_DIM]);
         }
 
-	complex float* maps = load_cfl(argv[2], DIMS, map_dims);
+	complex float* maps = load_cfl(sens_file, DIMS, map_dims);
 
 	unsigned int map_flags = md_nontriv_dims(DIMS, map_dims);
 
@@ -232,6 +259,7 @@ int main_pics(int argc, char* argv[])
 	if (NULL != basis_file) {
 
 		basis = load_cfl(basis_file, DIMS, basis_dims);
+
 		assert(!md_check_dimensions(DIMS, basis_dims, COEFF_FLAG | TE_FLAG));
 	}
 
@@ -275,13 +303,18 @@ int main_pics(int argc, char* argv[])
 	if ((NULL != traj_file) && (!md_check_compat(DIMS, ~0, ksp_dims, traj_dims)))
 		error("Dimensions of data and trajectory do not match!\n");
 
+	if ((NULL == traj_file) && (NULL != psf_ofile))
+		error("Export of PSF for Cartesian scan requested!\n");
 
 
 	assert(1 == ksp_dims[MAPS_DIM]);
 
 
 	if (conf.gpu)
-		num_init_gpu_device(gpun);
+		if (-1 == gpun)
+			num_init_gpu();
+		else
+			num_init_gpu_device(gpun);
 	else
 		num_init();
 
@@ -290,7 +323,7 @@ int main_pics(int argc, char* argv[])
 	if (conf.gpu)
 		debug_printf(DP_INFO, "GPU reconstruction\n");
 
-	if (map_dims[MAPS_DIM] > 1) 
+	if (map_dims[MAPS_DIM] > 1)
 		debug_printf(DP_INFO, "%ld maps.\nESPIRiT reconstruction.\n", map_dims[MAPS_DIM]);
 
 	if (conf.bpsense)
@@ -391,10 +424,9 @@ int main_pics(int argc, char* argv[])
 	}
 
 
-	// initialize forward_op and precond_op
+	// initialize forward_op
 
 	const struct linop_s* forward_op = NULL;
-	const struct operator_s* precond_op = NULL;
 
 	if (NULL == traj_file) {
 
@@ -410,8 +442,44 @@ int main_pics(int argc, char* argv[])
 
 	} else {
 
-		forward_op = sense_nc_init(max_dims, map_dims, maps, ksp_dims, traj_dims, traj, nuconf,
-				pat_dims, pattern, basis_dims, basis, (struct operator_s**)&precond_op);
+		const struct linop_s* nufft_op = NULL;
+
+		if ((NULL != psf_ifile) && (NULL == psf_ofile))
+			nuconf.nopsf = true;
+
+		forward_op = sense_nc_init(max_dims, map_dims, maps, ksp_dims,
+				traj_dims, traj, nuconf,
+				pat_dims, pattern,
+				basis_dims, basis, &nufft_op);
+
+		if (NULL != psf_ofile) {
+
+			int D = nufft_get_psf_dims(nufft_op, 0, NULL);
+
+			long psf_dims[D];
+
+			nufft_get_psf_dims(nufft_op, D, psf_dims);
+
+			complex float* psf_out = create_cfl(psf_ofile, D, psf_dims);
+
+			nufft_get_psf(nufft_op, D, psf_dims, psf_out);
+
+			unmap_cfl(D, psf_dims, psf_out);
+
+		}
+
+		if (NULL != psf_ifile) {
+
+			long psf_dims[DIMS + 1];
+
+			complex float* psf_in = load_cfl(psf_ifile, DIMS + 1, psf_dims);
+
+			nufft_update_psf(nufft_op, DIMS + 1, psf_dims, psf_in);
+
+			unmap_cfl(DIMS + 1, psf_dims, psf_in);
+		}
+
+		linop_free(nufft_op);
 	}
 
 
@@ -452,7 +520,7 @@ int main_pics(int argc, char* argv[])
 	}
 
 
-	complex float* image = create_cfl(argv[3], DIMS, img_dims);
+	complex float* image = create_cfl(out_file, DIMS, img_dims);
 	md_clear(DIMS, img_dims, image, CFL_SIZE);
 
 
@@ -480,7 +548,7 @@ int main_pics(int argc, char* argv[])
 	long img_start_dims[DIMS];
 	complex float* image_start = NULL;
 
-	if (warm_start) { 
+	if (warm_start) {
 
 		debug_printf(DP_DEBUG1, "Warm start: %s\n", image_start_file);
 
@@ -488,8 +556,6 @@ int main_pics(int argc, char* argv[])
 
 		assert(md_check_compat(DIMS, 0u, img_start_dims, img_dims));
 
-		xfree(image_start_file);
-
 		// if rescaling at the end, assume the input has also been rescaled
 		if (scale_im && (scaling != 0.))
 			md_zsmul(DIMS, img_dims, image_start, image_start, 1. / scaling);
@@ -529,7 +595,7 @@ int main_pics(int argc, char* argv[])
 
 	// FIXME: re-initialize forward_op and precond_op
 
-	if ((NULL == traj_file) && (0u != loop_flags) && !sms) { // FIXME: no basis
+	if ((NULL == traj_file) && ((0u != loop_flags && !sms) || conf.bpsense)) { // FIXME: no basis
 
 		linop_free(forward_op);
 		forward_op = sense_init(max1_dims, map_flags, maps);
@@ -557,18 +623,20 @@ int main_pics(int argc, char* argv[])
 	const struct operator_p_s* thresh_ops[NUM_REGS] = { NULL };
 	const struct linop_s* trafos[NUM_REGS] = { NULL };
 
-	opt_reg_configure(DIMS, img1_dims, &ropts, thresh_ops, trafos, llr_blk, shift_mode, conf.gpu);
+
+	opt_reg_configure(DIMS, img1_dims, &ropts, thresh_ops, trafos, llr_blk, shift_mode, wtype_str, conf.gpu);
 
 	if (conf.bpsense)
 		opt_bpursuit_configure(&ropts, thresh_ops, trafos, forward_op, kspace, bpsense_eps);
 
-	int nr_penalties = ropts.r;
-	struct reg_s* regs = ropts.regs;
+	int nr_penalties = ropts.r + ropts.sr;
+
+	debug_printf(DP_INFO, "Regularization terms: %d, Supporting variables: %d\n", nr_penalties, ropts.svars);
 
 	// choose algorithm
 
 	if (ALGO_DEFAULT == algo)
-		algo = italgo_choose(nr_penalties, regs);
+		algo = italgo_choose(ropts.r, ropts.regs);
 
 	if (conf.bpsense)
 		assert((ALGO_ADMM == algo) || (ALGO_PRIDU == algo));
@@ -599,7 +667,7 @@ int main_pics(int argc, char* argv[])
 
 	// initialize algorithm
 
-	struct iter it = italgo_config(algo, nr_penalties, regs, maxiter, step, hogwild, fast, admm, scaling, warm_start);
+	struct iter it = italgo_config(algo, nr_penalties, ropts.regs, maxiter, step, hogwild, fast, admm, scaling, warm_start);
 
 	if (ALGO_CG == algo)
 		nr_penalties = 0;
@@ -607,21 +675,60 @@ int main_pics(int argc, char* argv[])
 	bool trafos_cond = (   (ALGO_PRIDU == algo)
 			    || (ALGO_ADMM == algo)
 			    || (   (ALGO_NIHT == algo)
-				&& (regs[0].xform == NIHTWAV)));
+				&& (ropts.regs[0].xform == NIHTWAV)));
 
 	// FIXME: will fail with looped dims
 	struct iter_monitor_s* monitor = NULL;
 
 	if (im_truth)
-		monitor = create_monitor(2*md_calc_size(DIMS, img_dims), (const float*)image_truth, NULL, NULL); 
-	
+		monitor = iter_monitor_create(2 * md_calc_size(DIMS, img_dims), (const float*)image_truth, NULL, NULL);
+
+	if (0 < ropts.svars) {
+
+		assert(!im_truth);
+		assert(!conf.rvc);
+		assert(1 == img1_dims[BATCH_DIM]);
+		assert(1 == max1_dims[BATCH_DIM]);
+		assert(0 == loop_flags);
+
+		long img2_dims[DIMS];
+		md_copy_dims(DIMS, img2_dims, img1_dims);
+
+		img2_dims[BATCH_DIM] += ropts.svars;
+		long pos[DIMS] = { 0 };
+
+		max1_dims[BATCH_DIM] += ropts.svars;
+
+		forward_op = linop_chain_FF(
+				linop_extract_create(DIMS, pos, linop_domain(forward_op)->dims, img2_dims),
+				forward_op);
+	}
+
 	const struct operator_p_s* po = sense_recon_create(&conf, max1_dims, forward_op,
 				pat1_dims, ((NULL != traj_file) || conf.bpsense) ? NULL : pattern1,
 				it.italgo, it.iconf, image_start1, nr_penalties, thresh_ops,
-				trafos_cond ? trafos : NULL, precond_op, monitor);
+				trafos_cond ? trafos : NULL, NULL, monitor);
 
 	const struct operator_s* op = operator_p_bind(po, 1.);
-//	operator_p_free(po);	// FIXME
+	operator_p_free(po);
+
+	if (0 < ropts.svars) {
+
+		assert(img1_dims[BATCH_DIM] + ropts.svars == operator_codomain(op)->dims[BATCH_DIM]);
+
+		long pos[DIMS] = { 0 };
+
+		auto extr = linop_extract_create(DIMS, pos, img1_dims, operator_codomain(op)->dims);
+
+		auto op2 = operator_chain(op, extr->forward);
+
+		operator_free(op);
+		op = op2;
+
+		assert(img1_dims[BATCH_DIM] == operator_codomain(op)->dims[BATCH_DIM]);
+
+		linop_free(extr);
+	}
 
 	long strsx[2][DIMS];
 	const long* strs[2] = { strsx[0], strsx[1] };
@@ -629,7 +736,7 @@ int main_pics(int argc, char* argv[])
 	md_calc_strides(DIMS, strsx[0], img_dims, CFL_SIZE);
 	md_calc_strides(DIMS, strsx[1], ksp_dims, CFL_SIZE);
 
-	for (unsigned int i = 0; i < DIMS; i++) {
+	for (int i = 0; i < DIMS; i++) {
 
 		if (MD_IS_SET(loop_flags, i)) {
 
@@ -640,9 +747,14 @@ int main_pics(int argc, char* argv[])
 
 	if (0 != loop_flags) {
 
-		op = operator_copy_wrapper(2, strs, op);
+		auto op_tmp = operator_copy_wrapper(2, strs, op);
+		operator_free(op);
+		op = op_tmp;
+
 		// op = operator_loop(DIMS, loop_dims, op);
-		op = operator_loop_parallel(DIMS, loop_dims, op, loop_flags, conf.gpu);
+		op_tmp = operator_loop_parallel(DIMS, loop_dims, op, loop_flags, conf.gpu);
+		operator_free(op);
+		op = op_tmp;
 	}
 
 	operator_apply(op, DIMS, img_dims, image, DIMS, conf.bpsense ? img_dims : ksp_dims, conf.bpsense ? NULL : kspace);
@@ -687,15 +799,9 @@ int main_pics(int argc, char* argv[])
 	if (image_start)
 		unmap_cfl(DIMS, img_dims, image_start);
 
-	xfree(pat_file);
-	xfree(traj_file);
-	xfree(basis_file);
-
 	double end_time = timestamp();
 
 	debug_printf(DP_INFO, "Total Time: %f\n", end_time - start_time);
 
 	return 0;
 }
-
-
