@@ -260,6 +260,10 @@ void calc_geo_phantom(const long dims[DIMS], complex float* out, bool kspace, in
 		md_zadd(DIMS, dims, out, round, angular);
 		break;
 
+	case 3:
+		sample(dims, out, tstrs, traj, &(struct krn2d_data){ kspace, coeff, ARRAY_SIZE(phantom_geo5), phantom_geo5 }, krnX, kspace);
+		break;
+
 	default:
 		assert(0);
 	}
@@ -268,7 +272,7 @@ void calc_geo_phantom(const long dims[DIMS], complex float* out, bool kspace, in
 	md_free(angular);
 }
 
-static complex float cnst_one(void* _data, int s, const double mpos[2])
+static complex float cnst_one(void* _data, int s, const double mpos[3])
 {
 	UNUSED(_data);
 	UNUSED(mpos);
@@ -495,17 +499,41 @@ void calc_bart(const long dims[DIMS], complex float* out, bool kspace, const lon
 }
 
 
-
-void calc_phantom_arb(int N, const struct ellipsis_s data[N], const long dims[DIMS], complex float* out, bool kspace, const long tstrs[DIMS], const complex float* traj)
+void calc_phantom_arb(int N, const struct ellipsis_s data[N], const long dims[DIMS], complex float* out, bool kspace, const long tstrs[DIMS], const complex float* traj, float rotation_angle)
 {
 	bool coeff = (dims[COEFF_DIM] > 1);
 
 	assert((!coeff) || (0 == tstrs[COEFF_DIM]));
 	assert((!coeff) || (N == dims[COEFF_DIM]));
 
-	sample(dims, out, tstrs, traj, &(struct krn2d_data){ kspace, coeff, N, data }, krn2d, kspace);
-}
+	long strs[DIMS];
+	md_calc_strides(DIMS, strs, dims, sizeof(complex float));
 
+	long dims1[DIMS];
+	md_select_dims(DIMS, ~MD_BIT(TIME_DIM), dims1, dims);
+
+	for (int i = 1; i < dims[TIME_DIM]+1; i++) {
+
+		struct ellipsis_s data2[N];
+
+		complex float position = 0.;
+
+		for (int j = 0; j < N; j++) {
+
+			position = (data[j].center[0] + data[j].center[1]*I) * cexpf(-2.i * M_PI * rotation_angle / 360. * (float)i);
+
+			data2[j] = data[j];
+			data2[j].center[0] = crealf(position);
+			data2[j].center[1] = cimagf(position);
+			data2[j].axis[0] = data[j].axis[0];
+			data2[j].axis[1] = data[j].axis[1];
+		}
+
+		void* traj2 = (NULL == traj) ? NULL : ((void*)traj + (i - 1) * tstrs[TIME_DIM]);
+
+		sample(dims1, (void*)out + (i - 1) * strs[TIME_DIM], tstrs, traj2, &(struct krn2d_data){ kspace, coeff, N, data2 }, krn2d, kspace);
+	}
+}
 
 static void separate_bckgrd(int Nb, struct ellipsis_s bckgrd[Nb], int Nf, struct ellipsis_s frgrd[Nf], int N, const struct ellipsis_bs geometry[N])
 {
@@ -547,7 +575,7 @@ static bool circ_in_background(float s1, float px1, float py1, float s2, float p
 	return false;
 }
 
-void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace, bool random, int N, const long tstrs[DIMS], const complex float* traj)
+void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace, bool random, float rotation_angle, int N, const long tstrs[DIMS], const complex float* traj)
 {
 	if (1 < dims[COEFF_DIM]) {
 
@@ -636,9 +664,10 @@ void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace,
 			phantom_tubes_N[2 * N - 2] = _ebsb;
 		}
 		else {
-			assert(N == 11);
+			assert((8 == N) || (11 == N) || (15 == N));
+
 			for (int i = 0; i < 2 * N - 1; i++)
-				phantom_tubes_N[i] = phantom_tubes[i];
+				phantom_tubes_N[i] = (8 == N ? phantom_sonar : (15 == N ? nist_phantom_t2 : phantom_tubes))[i];
 		}
 
 
@@ -659,13 +688,25 @@ void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace,
 
 		complex float* bkgrd = md_alloc(DIMS, dims2, CFL_SIZE);
 
-		calc_phantom_arb(dims2[COEFF_DIM], tubes_bkgrd, dims2, bkgrd, kspace, tstrs, traj);
+		calc_phantom_arb(dims2[COEFF_DIM], tubes_bkgrd, dims2, bkgrd, kspace, tstrs, traj, rotation_angle);
 
-		// Save background to output
+		// Sum up all spatial coefficients
 
-		md_zsum(DIMS, dims2, COEFF_FLAG, out, bkgrd);
+		long dims3[DIMS];
+		md_select_dims(DIMS, ~COEFF_FLAG, dims3, dims);
+
+		complex float* tmp = md_alloc(DIMS, dims3, CFL_SIZE);
+
+		md_zsum(DIMS, dims2, COEFF_FLAG, tmp, bkgrd);
+
+		// Save summed up coefficients to out
+
+		long pos[DIMS] = { [0 ... DIMS - 1] = 0 };
+
+		md_copy_block(DIMS, pos, dims2, out, dims3, tmp, CFL_SIZE);
 
 		md_free(bkgrd);
+		md_free(tmp);
 
 		// Determine basis functions of the foreground
 
@@ -673,11 +714,10 @@ void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace,
 
 		complex float* frgrd = md_alloc(DIMS, dims2, CFL_SIZE);
 
-		calc_phantom_arb(dims2[COEFF_DIM], tubes_frgrd, dims2, frgrd, kspace, tstrs, traj);
+		calc_phantom_arb(dims2[COEFF_DIM], tubes_frgrd, dims2, frgrd, kspace, tstrs, traj, rotation_angle);
 
 		// Add foreground basis functions to out
 
-		long pos[DIMS] = { [0 ... DIMS - 1] = 0 };
 		pos[COEFF_DIM] = 1;
 
 		md_copy_block(DIMS, pos, dims, out, dims2, frgrd, CFL_SIZE);
@@ -693,7 +733,7 @@ void calc_phantom_tubes(const long dims[DIMS], complex float* out, bool kspace,
 
 		complex float* tmp = md_alloc(DIMS, tdims, CFL_SIZE);
 
-		calc_phantom_tubes(tdims, tmp, kspace, random, N, tstrs, traj);
+		calc_phantom_tubes(tdims, tmp, kspace, random, rotation_angle, N, tstrs, traj);
 
 		md_zsum(DIMS, tdims, COEFF_FLAG, out, tmp);
 		md_free(tmp);
