@@ -47,9 +47,14 @@ extern const struct ellipsis_s phantom_geo1[3];
 extern const struct ellipsis_s phantom_geo2[2];
 extern const struct ellipsis_s phantom_geo3[7];
 extern const struct ellipsis_s phantom_geo4[1];
+extern const struct ellipsis_s phantom_geo5[1];
 
 extern const struct ellipsis_bs phantom_tubes[21];
 
+extern const struct ellipsis_bs nist_phantom_t2[29];
+
+extern const struct ellipsis_bs phantom_sonar[15];
+
 
 extern complex double xellipsis(const double center[2], const double axis[2], double angle, const double p[2]);
 extern complex double kellipsis(const double center[2], const double axis[2], double angle, const double p[2]);
