@@ -1,4 +1,7 @@
 
+#ifndef _SIGNALS_H
+#define _SIGNALS_H 1
+
 #include <complex.h>
 #include <stdbool.h>
 
@@ -8,6 +11,7 @@ enum fat_spec {
 	FAT_SPEC_0,
 	FAT_SPEC_1,
 };
+
 struct signal_model {
 	
 	float m0;
@@ -26,6 +30,10 @@ struct signal_model {
 	bool ir;
 	bool ir_ss;
 	enum fat_spec fat_spec;
+        float time_T1relax;
+        long Hbeats;
+
+        int averaged_spokes;
 };
 
 
@@ -45,7 +53,7 @@ extern void looklocker_model(const struct signal_model* data, int N, complex flo
 
 extern void looklocker_model2(const struct signal_model* data, int N, complex float out[N]);
 
-extern void MOLLI_model(const struct signal_model* data, int N, int Hbeats, float time_T1relax, complex float out[N]);
+extern void MOLLI_model(const struct signal_model* data, int N, complex float out[N]);
 
 
 extern const struct signal_model signal_IR_bSSFP_defaults;
@@ -60,3 +68,6 @@ extern complex float calc_fat_modulation(float b0, float TE, enum fat_spec fs);
 
 extern void multi_grad_echo_model(const struct signal_model* data, int N, complex float out[N]);
 
+
+#endif // _SIGNALS_H
+
